/*
 * Copyright (c) 2017 Elvis Angelaccio <elvis.angelaccio@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KTOOLTIPWIDGET_TEST_H
#define KTOOLTIPWIDGET_TEST_H

#include <KToolTipWidget>

#include <QDialog>
#include <QTreeWidget>

class TestDialog : public QDialog
{
    Q_OBJECT

public:
    TestDialog(QWidget *parent = nullptr);

private:
    KToolTipWidget m_tooltipWidget;
    QTreeWidget m_view;
};

#endif
